@@ -19,13 +19,13 @@ var LibraryBrowser = {
   ],
   $Browser__postset: `
     // exports
-    Module["requestFullscreen"] = Browser.requestFullscreen;
+    Module['requestFullscreen'] = Browser.requestFullscreen;
 #if ASSERTIONS
-    Module["requestFullScreen"] = Browser.requestFullScreen;
+    Module['requestFullScreen'] = Browser.requestFullScreen;
 #endif
-    Module["setCanvasSize"] = Browser.setCanvasSize;
-    Module["getUserMedia"] = Browser.getUserMedia;
-    Module["createContext"] = Browser.createContext;
+    Module['setCanvasSize'] = Browser.setCanvasSize;
+    Module['getUserMedia'] = Browser.getUserMedia;
+    Module['createContext'] = Browser.createContext;
   `,
 
   $Browser: {
@@ -37,6 +37,8 @@ var LibraryBrowser = {
     preloadedImages: {},
     preloadedAudios: {},
 
+    getCanvas: () => Module['canvas'],
+
     init() {
       if (Browser.initted) return;
       Browser.initted = true;
@@ -153,12 +155,13 @@ var LibraryBrowser = {
       // Canvas event setup
 
       function pointerLockChange() {
-        Browser.pointerLock = document['pointerLockElement'] === Module['canvas'] ||
-                              document['mozPointerLockElement'] === Module['canvas'] ||
-                              document['webkitPointerLockElement'] === Module['canvas'] ||
-                              document['msPointerLockElement'] === Module['canvas'];
+        var canvas = Browser.getCanvas();
+        Browser.pointerLock = document['pointerLockElement'] === canvas ||
+                              document['mozPointerLockElement'] === canvas ||
+                              document['webkitPointerLockElement'] === canvas ||
+                              document['msPointerLockElement'] === canvas;
       }
-      var canvas = Module['canvas'];
+      var canvas = Browser.getCanvas();
       if (canvas) {
         // forced aspect ratio can be enabled by defining 'forcedAspectRatio' on Module
         // Module['forcedAspectRatio'] = 4 / 3;
@@ -182,8 +185,8 @@ var LibraryBrowser = {
 
         if (Module['elementPointerLock']) {
           canvas.addEventListener("click", (ev) => {
-            if (!Browser.pointerLock && Module['canvas'].requestPointerLock) {
-              Module['canvas'].requestPointerLock();
+            if (!Browser.pointerLock && Browser.getCanvas().requestPointerLock) {
+              Browser.getCanvas().requestPointerLock();
               ev.preventDefault();
             }
           }, false);
@@ -192,7 +195,7 @@ var LibraryBrowser = {
     },
 
     createContext(/** @type {HTMLCanvasElement} */ canvas, useWebGL, setInModule, webGLContextAttributes) {
-      if (useWebGL && Module['ctx'] && canvas == Module['canvas']) return Module['ctx']; // no need to recreate GL context if it's already been created for this canvas.
+      if (useWebGL && Module['ctx'] && canvas == Browser.getCanvas()) return Module['ctx']; // no need to recreate GL context if it's already been created for this canvas.
 
       var ctx;
       var contextHandle;
@@ -253,7 +256,7 @@ var LibraryBrowser = {
       if (typeof Browser.lockPointer == 'undefined') Browser.lockPointer = true;
       if (typeof Browser.resizeCanvas == 'undefined') Browser.resizeCanvas = false;
 
-      var canvas = Module['canvas'];
+      var canvas = Browser.getCanvas();
       function fullscreenChange() {
         Browser.isFullscreen = false;
         var canvasContainer = canvas.parentNode;
@@ -428,9 +431,8 @@ var LibraryBrowser = {
     calculateMouseCoords(pageX, pageY) {
       // Calculate the movement based on the changes
       // in the coordinates.
-      var rect = Module["canvas"].getBoundingClientRect();
-      var cw = Module["canvas"].width;
-      var ch = Module["canvas"].height;
+      var canvas = Browser.getCanvas();
+      var rect = canvas.getBoundingClientRect();
 
       // Neither .scrollX or .pageXOffset are defined in a spec, but
       // we prefer .scrollX because it is currently in a spec draft.
@@ -448,8 +450,8 @@ var LibraryBrowser = {
       // the canvas might be CSS-scaled compared to its backbuffer;
       // SDL-using content will want mouse coordinates in terms
       // of backbuffer units.
-      adjustedX = adjustedX * (cw / rect.width);
-      adjustedY = adjustedY * (ch / rect.height);
+      adjustedX = adjustedX * (canvas.width / rect.width);
+      adjustedY = adjustedY * (canvas.height / rect.height);
 
       return { x: adjustedX, y: adjustedY };
     },
@@ -508,12 +510,12 @@ var LibraryBrowser = {
     resizeListeners: [],
 
     updateResizeListeners() {
-      var canvas = Module['canvas'];
+      var canvas = Browser.getCanvas();
       Browser.resizeListeners.forEach((listener) => listener(canvas.width, canvas.height));
     },
 
     setCanvasSize(width, height, noUpdates) {
-      var canvas = Module['canvas'];
+      var canvas = Browser.getCanvas();
       Browser.updateCanvasDimensions(canvas, width, height);
       if (!noUpdates) Browser.updateResizeListeners();
     },
@@ -527,7 +529,7 @@ var LibraryBrowser = {
         flags = flags | 0x00800000; // set SDL_FULLSCREEN flag
         {{{ makeSetValue('SDL.screen', '0', 'flags', 'i32') }}};
       }
-      Browser.updateCanvasDimensions(Module['canvas']);
+      Browser.updateCanvasDimensions(Browser.getCanvas());
       Browser.updateResizeListeners();
     },
 
@@ -538,7 +540,7 @@ var LibraryBrowser = {
         flags = flags & ~0x00800000; // clear SDL_FULLSCREEN flag
         {{{ makeSetValue('SDL.screen', '0', 'flags', 'i32') }}};
       }
-      Browser.updateCanvasDimensions(Module['canvas']);
+      Browser.updateCanvasDimensions(Browser.getCanvas());
       Browser.updateResizeListeners();
     },
 
@@ -745,7 +747,7 @@ var LibraryBrowser = {
 
   emscripten_get_canvas_size__proxy: 'sync',
   emscripten_get_canvas_size: (width, height, isFullscreen) => {
-    var canvas = Module['canvas'];
+    var canvas = Browser.getCanvas();
     {{{ makeSetValue('width', '0', 'canvas.width', 'i32') }}};
     {{{ makeSetValue('height', '0', 'canvas.height', 'i32') }}};
     {{{ makeSetValue('isFullscreen', '0', 'Browser.isFullscreen ? 1 : 0', 'i32') }}};
