@@ -746,9 +746,9 @@ FS.staticInit();
     mkdirTree(path, mode) {
       var dirs = path.split('/');
       var d = '';
-      for (var i = 0; i < dirs.length; ++i) {
-        if (!dirs[i]) continue;
-        d += '/' + dirs[i];
+      for (var dir of dirs) {
+        if (!dir) continue;
+        d += '/' + dir;
         try {
           FS.mkdir(d, mode);
         } catch(e) {
@@ -1299,24 +1299,6 @@ FS.staticInit();
 #endif
       return bytesWritten;
     },
-    allocate(stream, offset, length) {
-      if (FS.isClosed(stream)) {
-        throw new FS.ErrnoError({{{ cDefs.EBADF }}});
-      }
-      if (offset < 0 || length <= 0) {
-        throw new FS.ErrnoError({{{ cDefs.EINVAL }}});
-      }
-      if ((stream.flags & {{{ cDefs.O_ACCMODE }}}) === {{{ cDefs.O_RDONLY}}}) {
-        throw new FS.ErrnoError({{{ cDefs.EBADF }}});
-      }
-      if (!FS.isFile(stream.node.mode) && !FS.isDir(stream.node.mode)) {
-        throw new FS.ErrnoError({{{ cDefs.ENODEV }}});
-      }
-      if (!stream.stream_ops.allocate) {
-        throw new FS.ErrnoError({{{ cDefs.EOPNOTSUPP }}});
-      }
-      stream.stream_ops.allocate(stream, offset, length);
-    },
     mmap(stream, length, position, prot, flags) {
       // User requests writing to file (prot & PROT_WRITE != 0).
       // Checking if we have permissions to write to the file unless
@@ -1568,12 +1550,10 @@ FS.staticInit();
       _fflush(0);
 #endif
       // close all of our streams
-      for (var i = 0; i < FS.streams.length; i++) {
-        var stream = FS.streams[i];
-        if (!stream) {
-          continue;
+      for (var stream of FS.streams) {
+        if (stream) {
+          FS.close(stream);
         }
-        FS.close(stream);
       }
     },
 
