@@ -190,7 +190,7 @@ class Ports:
           if ext in ('.c', '.cpp') and not any((excluded in f) for excluded in exclude_files):
             srcs.append(os.path.join(root, f))
 
-    cflags = system_libs.get_base_cflags() + ['-O2', '-I' + src_dir] + flags
+    cflags = system_libs.get_base_cflags(build_dir) + ['-O2', '-I' + src_dir] + flags
     for include in includes:
       cflags.append('-I' + include)
 
